"""
ttop is CUI graphical system monitr.
"""

__author__ = "ton1517"
__email__ = "tonton1517@gmail.com"
__homepage__ = "https://github.com/ton1517/ttop"
__version__ = "0.6.2"
__license__ = "MIT License"
